@@ -1,6 +1,6 @@
 /* conf-manager.c
  *
- * Copyright (C) 2001 - 2002 Vivien Malerba
+ * Copyright (C) 2001 - 2003 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,6 +30,7 @@
 #include "marshal.h"
 #include "object-selector.h"
 #include "item-selector.h"
+#include "settings.h"
 
 static void conf_manager_class_init (ConfManagerClass * class);
 static void conf_manager_init (ConfManager * conf);
@@ -152,6 +153,21 @@ conf_manager_class_init (ConfManagerClass * class)
 	object_class->finalize = conf_manager_finalize;
 }
 
+static void
+config_notification_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	ConfManager *conf = (ConfManager *) user_data;
+
+	if (server_access_is_open (conf->srv)) {
+		if (!strcmp (entry->key, ENTRY_SHOW_SHORTCUT_BAR)) {
+			if (settings_get_show_shortcut_bar ())
+				gtk_widget_show (conf->shortcut_bar);
+			else
+				gtk_widget_hide (conf->shortcut_bar);
+		}
+	}
+}
+
 static void
 conf_manager_init (ConfManager * conf)
 {
@@ -160,11 +176,17 @@ conf_manager_init (ConfManager * conf)
 	conf->nb = NULL;
 	conf->working_box = NULL;
 	conf->title_bar = NULL;
+	conf->icon_area = NULL;
+	conf->icon_area_tooltips = NULL;
+	conf->icon_current_page = NULL;
+	conf->icon_transaction = NULL;
 	conf->error_dlg = NULL;
 	conf->manager_bonobo_win = NULL;
 	conf->relations_dialog = NULL;
 	conf->config_plugins_dlg = NULL;
 
+	conf->shortcut_sequences = NULL;
+
 	conf->refresh_status_dlg = NULL;
 
 	conf->tables_page = NULL;
@@ -180,6 +202,7 @@ conf_manager_init (ConfManager * conf)
 	conf->conn_open_requested = FALSE;
 	conf->top_query = NULL;
 	conf->id_serial = 0;
+	conf->current_transaction = NULL;
 
 	conf->working_file = NULL;
 	conf->save_up_to_date = TRUE;
@@ -210,6 +233,11 @@ conf_manager_init (ConfManager * conf)
 	conf->clipboard_obj = NULL;
 
 	conf->created_envs = NULL;
+
+	/* install configuration listener */
+	conf->config_listener = gnome_db_config_add_listener ("/apps/mergeant",
+							      (GConfClientNotifyFunc) config_notification_cb,
+							      conf);
 }
 
 static void widgets_conn_management_cb (ServerAccess *srv, ConfManager *conf);
@@ -252,8 +280,8 @@ conf_manager_finish_prepare (ConfManager *conf)
 			  G_CALLBACK (sql_server_conn_to_close_cb), conf);
 	g_signal_connect (G_OBJECT (conf->srv), "conn_closed",
 			  G_CALLBACK (sql_server_conn_close_cb), conf);
-	g_signal_connect (G_OBJECT (conf->srv), "error",
-			  G_CALLBACK (sql_server_catch_errors_cb), conf);
+	g_signal_connect (G_OBJECT (conf->srv), "event_notification",
+			  G_CALLBACK (sql_server_event_cb), conf);
 
 	/* those 2 CBs are to show/hide and order sensitiveness of objects */
 	g_signal_connect (G_OBJECT (conf->srv), "conn_opened",
@@ -272,6 +300,7 @@ static gint register_icon (gchar * name, gchar * filename);
 static void add_toolbar (GnomeApp * app, ConfManager * conf);
 static void add_menus (GnomeApp * app, ConfManager * conf);
 static void todo_cb (GtkWidget * widget, ConfManager * conf);
+static void help_contents_cb (GtkWidget * widget, ConfManager * conf);
 static void about_cb (GtkWidget * widget, ConfManager * conf);
 static gint delete_event_cb (GtkWidget * wid, GdkEvent * event, ConfManager * conf);
 #ifdef debug
@@ -285,15 +314,19 @@ static void debug_dump_queries_cb (GtkWidget * widget, ConfManager * conf);
 static void 
 build_main_window (ConfManager *conf)
 {
-	GtkWidget *vb, *wid, *owid, *hb, *sc;
+	GtkWidget *vb, *wid, *owid, *hb;
 	gchar *str;
 
-
 	str = conf_manager_get_title (conf);
 	conf->app = gnome_app_new ("Mergeant", str);
 	gnome_window_icon_set_from_file (GTK_WINDOW (conf->app), PIXMAPDIR "/mergeant.png");
 	g_free (str);
-	gtk_widget_set_usize (conf->app, 600, 388);
+	if (settings_get_remember_window_size ()) {
+		gtk_widget_set_size_request (conf->app,
+					     settings_get_window_width (),
+					     settings_get_window_height ());
+	} else
+		gtk_widget_set_size_request (conf->app, 600, 388);
 
 	conf->appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_USER);
 	gnome_appbar_set_default (GNOME_APPBAR (conf->appbar), _("Ready."));
@@ -330,9 +363,10 @@ build_main_window (ConfManager *conf)
 	gtk_box_pack_start (GTK_BOX (vb), hb, TRUE, TRUE, 0);
 
 	/* shortcut part */
-	sc = gnome_db_shortcut_new ();
-	gtk_widget_show (sc);
-	gnome_db_shortcut_append (GNOME_DB_SHORTCUT (sc),
+	conf->shortcut_bar = gnome_db_shortcut_new ();
+	if (settings_get_show_shortcut_bar ())
+		gtk_widget_show (conf->shortcut_bar);
+	gnome_db_shortcut_append (GNOME_DB_SHORTCUT (conf->shortcut_bar),
 				  _("Tables & Views"),
 				  _("Work on tables and views"),
 				  GNOME_DB_STOCK_TABLES,
@@ -342,17 +376,19 @@ build_main_window (ConfManager *conf)
 		str = "mergeant_sequences";
 	else
 		str = GTK_STOCK_DIALOG_INFO;
-	gnome_db_shortcut_append (GNOME_DB_SHORTCUT (sc),
-				  _("Sequences"),
-				  _("Work on sequences"),
-				  str,
-				  (GnomeDbShortcutSelectFunc) show_seqs_page_cb, conf);
-
+        wid = gnome_db_shortcut_append (GNOME_DB_SHORTCUT (conf->shortcut_bar),
+                                        _("Sequences"),
+                                        _("Work on sequences"),
+                                        str,
+                                        (GnomeDbShortcutSelectFunc) show_seqs_page_cb, conf);
+	  
+	conf->shortcut_sequences = wid;
+         
 	if (register_icon ("mergeant_queries", "mergeant_queries.png"))
 		str = "mergeant_queries";
 	else
 		str = GTK_STOCK_DIALOG_INFO;
-	gnome_db_shortcut_append (GNOME_DB_SHORTCUT (sc),
+	gnome_db_shortcut_append (GNOME_DB_SHORTCUT (conf->shortcut_bar),
 				  _("Queries"),
 				  _("Manage queries"),
 				  str,
@@ -362,41 +398,61 @@ build_main_window (ConfManager *conf)
 		str = "mergeant_forms";
 	else
 		str = GTK_STOCK_DIALOG_INFO;
-	gnome_db_shortcut_append (GNOME_DB_SHORTCUT (sc),
+	gnome_db_shortcut_append (GNOME_DB_SHORTCUT (conf->shortcut_bar),
 				  _("Forms"),
 				  _("Manage forms"),
 				  str,
 				  (GnomeDbShortcutSelectFunc) show_forms_page_cb, conf);
 
 
-	gnome_db_shortcut_append (GNOME_DB_SHORTCUT (sc),
+	gnome_db_shortcut_append (GNOME_DB_SHORTCUT (conf->shortcut_bar),
 				  _("SQL"),
 				  _("Enter SQL commands"),
 				  GNOME_DB_STOCK_SQL,
 				  (GnomeDbShortcutSelectFunc) show_sql_page_cb, conf);
 
-	gtk_box_pack_start (GTK_BOX (hb), sc, FALSE, FALSE, 0);
-	gtk_widget_show (sc);
+	gtk_box_pack_start (GTK_BOX (hb), conf->shortcut_bar, FALSE, FALSE, 0);
 
 	/* title bar and notebook */
-	wid = gtk_table_new (2, 1, FALSE);
+	wid = gtk_table_new (2, 2, FALSE);
 	gtk_widget_show (wid);
 	gtk_box_pack_start (GTK_BOX (hb), wid, TRUE, TRUE, 0);
 
 	conf->title_bar = gnome_db_gray_bar_new (_("Tables and Views"));
 	gtk_widget_show (conf->title_bar);
 	gtk_table_attach (GTK_TABLE (wid), conf->title_bar, 0, 1, 0, 1,
-			  GTK_FILL, GTK_FILL, 3, 3);
+			  GTK_FILL | GTK_EXPAND | GTK_SHRINK,
+			  GTK_FILL, 3, 3);
 
 	conf->nb = gtk_notebook_new ();
+	gtk_notebook_set_show_border (GTK_NOTEBOOK (conf->nb), FALSE);
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (conf->nb), FALSE);
-	gtk_table_attach (GTK_TABLE (wid), conf->nb, 0, 1, 1, 2,
+	gtk_table_attach (GTK_TABLE (wid), conf->nb, 0, 2, 1, 2,
 			  GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 			  GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 			  1, 1);
 	gtk_widget_show (conf->nb);
 	gtk_widget_set_usize (GTK_WIDGET (conf->nb), 420, 270);
 
+	/* icon status area */
+	conf->icon_area = gnome_db_new_hbox_widget (FALSE, 2);
+	gtk_table_attach (GTK_TABLE (wid), conf->icon_area, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
+
+	conf->icon_area_tooltips = gtk_tooltips_new ();
+	gtk_tooltips_enable (GTK_TOOLTIPS (conf->icon_area_tooltips));
+
+	conf->icon_current_page = gtk_image_new_from_stock (GNOME_DB_STOCK_TABLES, GTK_ICON_SIZE_MENU);
+	gtk_widget_show (conf->icon_current_page);
+	gtk_box_pack_start (GTK_BOX (conf->icon_area), conf->icon_current_page, FALSE, FALSE, 2);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_current_page,
+			      _("Tables and Views page"), _("Tables and Views page"));
+
+	conf->icon_transaction = gtk_image_new_from_stock (GNOME_DB_STOCK_NO_TRANSACTION, GTK_ICON_SIZE_MENU);
+	gtk_widget_show (conf->icon_transaction);
+	gtk_box_pack_start (GTK_BOX (conf->icon_area), conf->icon_transaction, FALSE, FALSE, 2);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_transaction,
+			      _("No transaction"), _("No transaction"));
+
 	/* Tables */
 	owid = main_page_table_new (conf);
 	gtk_widget_show (owid);
@@ -424,16 +480,11 @@ build_main_window (ConfManager *conf)
 	gtk_widget_show (conf->app);
 
 	/* Forms */
-	vb = gtk_vbox_new (FALSE, 0);
-/* 	wid = gtk_label_new ("This is a test page displaying another instance\n" */
-/* 			     "of the queries list page; the two pages can be used at the\n" */
-/* 			     "same time, and still be both updated!"); */
-/* 	gtk_box_pack_start (GTK_BOX (vb), wid, FALSE, FALSE, GNOME_PAD/2.); */
-	owid = main_page_env_new (conf); 
- 	gtk_box_pack_start (GTK_BOX (vb), owid, TRUE, TRUE, GNOME_PAD/2.); 
-	gtk_widget_show_all (vb);
 	wid = gtk_label_new (_("Forms"));
-	gtk_notebook_append_page (GTK_NOTEBOOK (conf->nb), vb, wid);
+	gtk_widget_show (wid);
+	owid = main_page_env_new (conf); 
+	gtk_widget_show (owid);
+	gtk_notebook_append_page (GTK_NOTEBOOK (conf->nb), owid, wid);
 
 	/* SQL editor */
 	wid = gtk_label_new (_("SQL"));
@@ -493,6 +544,7 @@ register_icon (gchar * name, gchar * filename)
 		str = g_strdup_printf ("%s/%s", PIXMAPDIR, filename);
 
 	pixbuf = gdk_pixbuf_new_from_file (str, NULL);
+	g_free (str);
 	if (pixbuf) {
 		iconf = gtk_icon_factory_new ();
 		icons = gtk_icon_set_new_from_pixbuf (pixbuf);
@@ -527,6 +579,28 @@ add_toolbar (GnomeApp * app, ConfManager * conf)
 			sql_show_relations_cb, conf, NULL,
 			GNOME_APP_PIXMAP_STOCK, relations_icon,
 			0, 0, NULL},
+		GNOMEUIINFO_SEPARATOR,
+		{
+			GNOME_APP_UI_ITEM, N_("Commit transaction"),
+			N_("Commit current transaction"),
+			sql_commit_trans_cb,
+			conf, NULL,
+			GNOME_APP_PIXMAP_STOCK, GTK_STOCK_APPLY,
+			0, 0, NULL},
+		{
+			GNOME_APP_UI_ITEM, N_("Rollback transaction"),
+			N_("Cancel current transaction"),
+			sql_rollback_trans_cb,
+			conf, NULL,
+			GNOME_APP_PIXMAP_STOCK, GTK_STOCK_REVERT_TO_SAVED, NULL,
+			0, 0, NULL},
+		GNOMEUIINFO_SEPARATOR,
+		{
+			GNOME_APP_UI_ITEM, N_("_Contents"),
+			N_("Display Mergeant help"),
+			help_contents_cb, conf,
+			NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_HELP,
+			0, 0, NULL},
                 GNOMEUIINFO_END
         };
 	
@@ -534,8 +608,10 @@ add_toolbar (GnomeApp * app, ConfManager * conf)
 	/* widgets to be shown or hidden depending on the connexion state */
 	conf_manager_register_show_on_connect (conf, (toolbar[1]).widget);
 	conf_manager_register_show_on_disconnect (conf, (toolbar[0]).widget);
-	/*conf_manager_register_sensitive_on_disconnect(conf, (toolbar[2]).widget); */
-	conf_manager_register_sensitive_on_connect (conf, (toolbar[2]).widget);	/*3! */
+	conf_manager_register_sensitive_on_connect (conf, (toolbar[2]).widget);
+	/* toolbar[3] is a separator */
+	conf_manager_register_sensitive_on_connect (conf, (toolbar[4]).widget);
+	conf_manager_register_sensitive_on_connect (conf, (toolbar[5]).widget);
 }
 
 
@@ -546,9 +622,8 @@ add_menus (GnomeApp * app, ConfManager * conf)
 	/* file menu */
 	GnomeUIInfo file_menu[] = {
 		GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Workspace"),
-                                           N_
-                                           ("Switch to a new workspace to work on "
-                                            "another database"), file_new_cb,
+                                           N_("Switch to a new workspace to work on "
+					      "another database"), file_new_cb,
                                            conf),
                 GNOMEUIINFO_MENU_OPEN_ITEM (file_open_cb, conf),
                 GNOMEUIINFO_MENU_CLOSE_ITEM (file_close_cb, conf),
@@ -578,13 +653,13 @@ add_menus (GnomeApp * app, ConfManager * conf)
 		GNOMEUIINFO_SEPARATOR,
 		{
 			GNOME_APP_UI_ITEM, N_("Plugins..."),
-			N_("Set up Mergeant preferences"),
+			N_("Configure Mergeant plugins"),
 			config_display_plugins_cb, conf,
 			NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PREFERENCES,
 			0, 0, NULL},
 		{
-			GNOME_APP_UI_ITEM, N_("Database Manager..."),
-			N_("The GNOME-DB manager"),
+			GNOME_APP_UI_ITEM, N_("Data source configuration..."),
+			N_("Manage data sources"),
 			run_gnomedb_manager_cb, conf,
 			NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_INDEX,
 			0, 0, NULL},
@@ -593,45 +668,82 @@ add_menus (GnomeApp * app, ConfManager * conf)
 
 	/* help menu */
         GnomeUIInfo help_menu[] = {
-                /*GNOMEUIINFO_HELP (_("Mergeant")),FIXME: REMOVE COMMENTS*/
-                GNOMEUIINFO_SEPARATOR,
+                {
+			GNOME_APP_UI_ITEM, N_("_Contents"),
+			N_("Display Mergeant help"),
+			help_contents_cb, conf,
+			NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_HELP,
+			0, 0, NULL},
                 GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, conf),
                 GNOMEUIINFO_END
         };
 
-        /* connection menu */
-        GnomeUIInfo conn_menu[] = {
+	/* view menu */
+	GnomeUIInfo view_menu[] = {
+		{
+			GNOME_APP_UI_ITEM, N_("_Tables and Views"),
+			N_("Work on tables and views"),
+			view_tables_tab_cb, conf,
+			NULL, GNOME_APP_PIXMAP_STOCK, GNOME_DB_STOCK_TABLES,
+			't', GDK_CONTROL_MASK, NULL},
+		{
+			GNOME_APP_UI_ITEM, N_("S_equences"),
+			N_("Work on sequences"),
+			view_sequences_tab_cb, conf,
+			NULL, GNOME_APP_PIXMAP_STOCK, "mergeant_sequences",
+			'e', GDK_CONTROL_MASK, NULL},
+		{
+			GNOME_APP_UI_ITEM, N_("Que_ries"),
+			N_("Manage queries"),
+			view_queries_tab_cb, conf,
+			NULL, GNOME_APP_PIXMAP_STOCK, "mergeant_queries",
+			'r', GDK_CONTROL_MASK, NULL},
+		{
+			GNOME_APP_UI_ITEM, N_("_Forms"),
+			N_("Manage forms"),
+			view_forms_tab_cb, conf,
+			'f', GNOME_APP_PIXMAP_STOCK, "mergeant_forms",
+			0, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL},
+		{
+			GNOME_APP_UI_ITEM, N_("SQ_L"),
+			N_("Enter SQL commands"),
+			view_sql_tab_cb, conf,
+			NULL, GNOME_APP_PIXMAP_STOCK, GNOME_DB_STOCK_SQL,
+			'l', GDK_CONTROL_MASK, NULL},
+		GNOMEUIINFO_END
+	};
+
+        /* database menu */
+        GnomeUIInfo db_menu[] = {
                 {
-			GNOME_APP_UI_ITEM, N_("Connect Sql server"), NULL,
+			GNOME_APP_UI_ITEM, N_("Connect Sql server"),
+			N_("Open connection to database server"),
 			sql_conn_open_cb,
 			conf, NULL,
 			GNOME_APP_PIXMAP_STOCK, connect_icon_small, /*GTK_STOCK_EXECUTE,*/
 			0, 0, NULL},
                 {
-			GNOME_APP_UI_ITEM, N_("Disconnect Sql server"), NULL,
+			GNOME_APP_UI_ITEM, N_("Disconnect Sql server"),
+			N_("Close active connection"),
 			sql_conn_close_cb,
 			conf, NULL,
 			GNOME_APP_PIXMAP_STOCK, disconnect_icon_small, /*GTK_STOCK_EXECUTE,*/
 			0, 0, NULL},
-                GNOMEUIINFO_END
-        };
-
-        /* database menu */
-        GnomeUIInfo db_menu[] = {
-                {
-			GNOME_APP_UI_ITEM, N_("Relations Scheme"), NULL,
+		GNOMEUIINFO_SEPARATOR,
+		{
+			GNOME_APP_UI_ITEM, N_("Relations Scheme"),
+			N_("Display database relations scheme"),
 			sql_show_relations_cb,
 			conf, NULL,
 			GNOME_APP_PIXMAP_STOCK, relations_icon_small,
 			0, 0, NULL},
-                GNOMEUIINFO_SEPARATOR,
                 {
-			GNOME_APP_UI_ITEM, N_("Integrity check"), NULL,
+			GNOME_APP_UI_ITEM, N_("Integrity check"),
+			N_("Perform database integrity check"),
 			NULL /*integrity_check_cb*/,
 			conf, NULL,
 			GNOME_APP_PIXMAP_NONE, NULL,
 			0, 0, NULL},
-                GNOMEUIINFO_SEPARATOR,
                 {
 			GNOME_APP_UI_ITEM, N_("System informations"),
 			N_("View the data types, functions, "
@@ -642,7 +754,7 @@ add_menus (GnomeApp * app, ConfManager * conf)
                 {
 			GNOME_APP_UI_ITEM, N_("Refresh memory structure"),
 			N_("Updates the memory "
-			   "representation\nof the database's structure"),
+			   "representation of the database's structure"),
 			sql_mem_update_cb,
 			conf, NULL,
 			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_REFRESH,
@@ -653,6 +765,25 @@ add_menus (GnomeApp * app, ConfManager * conf)
 			conf, NULL,
 			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_NEW,
 			0, 0, NULL},
+		GNOMEUIINFO_SEPARATOR,
+		{
+			GNOME_APP_UI_ITEM, N_("Begin transaction"), N_("Start a new transaction"),
+			sql_begin_trans_cb,
+			conf, NULL,
+			GNOME_APP_PIXMAP_NONE, NULL,
+			0, 0, NULL},
+		{
+			GNOME_APP_UI_ITEM, N_("Commit transaction"), N_("Commit current transaction"),
+			sql_commit_trans_cb,
+			conf, NULL,
+			GNOME_APP_PIXMAP_STOCK, GTK_STOCK_APPLY,
+			0, 0, NULL},
+		{
+			GNOME_APP_UI_ITEM, N_("Rollback transaction"), N_("Cancel current transaction"),
+			sql_rollback_trans_cb,
+			conf, NULL,
+			GNOME_APP_PIXMAP_STOCK, GTK_STOCK_REVERT_TO_SAVED, NULL,
+			0, 0, NULL},
                 GNOMEUIINFO_END
         };
 
@@ -697,7 +828,7 @@ add_menus (GnomeApp * app, ConfManager * conf)
         /* ALL menus */
         GnomeUIInfo main_menu[] = {
                 GNOMEUIINFO_MENU_FILE_TREE (file_menu),
-                GNOMEUIINFO_SUBTREE (N_("_Connection"), &conn_menu),
+		GNOMEUIINFO_SUBTREE (N_("_View"), &view_menu),
                 GNOMEUIINFO_SUBTREE (N_("_Database"), &db_menu),
                 GNOMEUIINFO_MENU_SETTINGS_TREE (&settings_menu),
 #ifdef debug
@@ -709,20 +840,28 @@ add_menus (GnomeApp * app, ConfManager * conf)
 
 	gnome_app_create_menus (app, main_menu);
 
-	/* widgets to be shown or hidden depending on the connexion state */
-	conf_manager_register_show_on_connect (conf, (db_menu[0]).widget);
+	/* widgets to be shown or hidden depending on the connection state */
+	conf_manager_register_show_on_connect (conf, (view_menu[0]).widget);
+	conf_manager_register_show_on_connect (conf, (view_menu[1]).widget);
+	conf_manager_register_show_on_connect (conf, (view_menu[2]).widget);
+	conf_manager_register_show_on_connect (conf, (view_menu[3]).widget);
+	conf_manager_register_show_on_connect (conf, (view_menu[4]).widget);
+	conf_manager_register_show_on_disconnect (conf, (db_menu[0]).widget);
 	conf_manager_register_show_on_connect (conf, (db_menu[1]).widget);
-	conf_manager_register_show_on_connect (conf, (db_menu[2]).widget);
 	conf_manager_register_show_on_connect (conf, (db_menu[3]).widget);
 	conf_manager_register_show_on_connect (conf, (db_menu[4]).widget);
 	conf_manager_register_show_on_connect (conf, (db_menu[5]).widget);
-	conf_manager_register_show_on_disconnect (conf, (db_menu[6]).widget);
-
-	conf_manager_register_sensitive_on_disconnect (conf, (conn_menu[0]).widget);
-	conf_manager_register_sensitive_on_connect (conf, (conn_menu[1]).widget);
+	conf_manager_register_show_on_connect (conf, (db_menu[6]).widget);
+	conf_manager_register_show_on_disconnect (conf, (db_menu[7]).widget);
+	conf_manager_register_show_on_connect (conf, (db_menu[8]).widget);
+	conf_manager_register_show_on_connect (conf, (db_menu[9]).widget);
+	conf_manager_register_show_on_connect (conf, (db_menu[10]).widget);
+	conf_manager_register_show_on_connect (conf, (db_menu[11]).widget);
 	conf_manager_register_sensitive_on_disconnect (conf, (file_menu[1]).widget);
 	conf_manager_register_sensitive_on_disconnect (conf, (file_menu[2]).widget);
 	conf_manager_register_sensitive_on_connect (conf, (file_menu[5]).widget);
+
+	gnome_app_install_appbar_menu_hints (GNOME_APPBAR (conf->appbar), main_menu);
 }
 
 #ifdef debug
@@ -957,6 +1096,18 @@ todo_cb (GtkWidget * widget, ConfManager * conf)
 				  GTK_WINDOW (conf->app));
 }
 
+static void
+help_contents_cb (GtkWidget * widget, ConfManager * conf)
+{
+	GError *error = NULL;
+
+	gnome_help_display ("index.html", NULL, &error);
+	if (error) {
+		gnome_db_show_error (error->message);
+		g_error_free (error);
+	}
+}
+
 static void
 about_cb (GtkWidget * widget, ConfManager * conf)
 {
@@ -995,14 +1146,14 @@ about_cb (GtkWidget * widget, ConfManager * conf)
 			"Kjartan Maraas <kmaraas@online.no> Norwegian translation\n" \
 			"Zbigniew Chyla <translators@gnome.pl> Polish translation\n" \
 			"Pablo del Campo <pablodc@bigfoot.com> Spanish translation\n" \
-			"Christophe Merlet (RedFox) <redfox@eikonex.org> French translation\n" \
+			"Christophe Merlet (RedFox) <redfox@redfoxcenter.org> French translation\n" \
 			"Trinh Minh Thanh <tmthanh@linuxmail.org> Vietnamese translation\n";
 
 		/* FIXME: use a real GError here */
                 icon = gdk_pixbuf_new_from_file (PIXMAPDIR "/mergeant.png", NULL);
 		
 		dialog = gnome_about_new (_("Mergeant"), VERSION,
-					  "(C) 1999-2002 Vivien Malerba",
+					  "(C) 1999-2003 Vivien Malerba",
 					  _("A Database admin tool for any SQL database "
 					   "accessible with the gnome-db module."),
 					  authors,
@@ -1025,6 +1176,15 @@ delete_event_cb (GtkWidget * wid, GdkEvent * event, ConfManager * conf)
 	/* quit with confirmation */
 	quit_cb (NULL, conf);
 
+	/* save configuration */
+	if (settings_get_remember_window_size ()) {
+		gint x, y, width, height;
+
+		gdk_window_get_geometry (GTK_WIDGET (conf->app)->window, &x, &y, &width, &height, NULL);
+		settings_set_window_width (width);
+		settings_set_window_height (height);
+	}
+
 	/* we don't want the "destroy" signal to be emitted now */
 	return TRUE;
 }
@@ -1176,24 +1336,25 @@ static void query_dropped_cb (Query *top_query, Query *new_query, ConfManager *
 static void 
 objects_conn_management_cb (ServerAccess *srv, ConfManager *conf)
 {
-	if (server_access_is_open (conf->srv)) {
+	if (server_access_is_open (conf->srv)) { /* the connection is now opened */
 		guint id;
 
 		g_assert (!conf->db);
 		conf->db = DATABASE (database_new (conf->srv));
 
 		g_signal_connect (G_OBJECT (conf->db), "fault", 
-				    G_CALLBACK (access_db_link_cb), conf); 
+				  G_CALLBACK (access_db_link_cb), conf); 
 		g_signal_connect (G_OBJECT (conf->db), "updated",
-				    G_CALLBACK (conf_update_save_flag),
+				  G_CALLBACK (conf_update_save_flag),
 				    conf);
 		g_signal_connect (G_OBJECT (conf->db), "progress",
-				    G_CALLBACK (progress_cb), conf);
+				  G_CALLBACK (progress_cb), conf);
 
 #ifdef debug_signal
 		g_print (">> 'DATABASE_ADDED' from objects_conn_management_cb\n");
 #endif
-		g_signal_emit (G_OBJECT (conf), conf_manager_signals[DATABASE_ADDED], 0, conf->db);
+		g_signal_emit (G_OBJECT (conf), conf_manager_signals[DATABASE_ADDED], 0, 
+			       conf->db);
 #ifdef debug_signal
 		g_print ("<< 'DATABASE_ADDED' from objects_conn_management_cb\n");
 #endif	
@@ -1207,21 +1368,29 @@ objects_conn_management_cb (ServerAccess *srv, ConfManager *conf)
 #ifdef debug_signal
 		g_print (">> 'QUERY_ADDED' from objects_conn_management_cb\n");
 #endif
-		g_signal_emit (G_OBJECT (conf), conf_manager_signals[QUERY_ADDED], 0, conf->top_query);
+		g_signal_emit (G_OBJECT (conf), conf_manager_signals[QUERY_ADDED], 0, 
+			       conf->top_query);
 #ifdef debug_signal
 		g_print ("<< 'QUERY_ADDED' from objects_conn_management_cb\n");
 #endif	
 		g_signal_connect (G_OBJECT (conf->top_query), "query_created",
-				    G_CALLBACK (query_created_cb), conf);
+				  G_CALLBACK (query_created_cb), conf);
 		g_signal_connect (G_OBJECT (conf->top_query), "query_dropped",
-				    G_CALLBACK (query_dropped_cb), conf);
+				  G_CALLBACK (query_dropped_cb), conf);
+
+		/* Hide or show some shortcut icons depending on the DBMS features */
+		if (conf->srv->features.sequences) 
+		        gtk_widget_show (conf->shortcut_sequences);
+		else
+		        gtk_widget_hide (conf->shortcut_sequences);
 	}
-	else {
+	else { /* the connection is now closed */
 		g_assert (conf->db);
 #ifdef debug_signal
 		g_print (">> 'DATABASE_REMOVED' from objects_conn_management_cb\n");
 #endif
-		g_signal_emit (G_OBJECT (conf), conf_manager_signals[DATABASE_REMOVED], 0, conf->db);
+		g_signal_emit (G_OBJECT (conf), conf_manager_signals[DATABASE_REMOVED], 0, 
+			       conf->db);
 #ifdef debug_signal
 		g_print ("<< 'DATABASE_REMOVED' from objects_conn_management_cb\n");
 #endif	
@@ -1232,7 +1401,8 @@ objects_conn_management_cb (ServerAccess *srv, ConfManager *conf)
 #ifdef debug_signal
 		g_print (">> 'QUERY_REMOVED' from objects_conn_management_cb\n");
 #endif
-		g_signal_emit (G_OBJECT (conf), conf_manager_signals[QUERY_REMOVED], 0, conf->top_query);
+		g_signal_emit (G_OBJECT (conf), conf_manager_signals[QUERY_REMOVED], 0, 
+			       conf->top_query);
 #ifdef debug_signal
 		g_print ("<< 'QUERY_REMOVED' from objects_conn_management_cb\n");
 #endif			
@@ -1429,6 +1599,8 @@ conf_manager_finalize (GObject   * object)
 
 	conf = CONF_MANAGER (object);
 
+	gnome_db_config_remove_listener (conf->config_listener);
+
 	/* for the parent class */
 	parent_class->finalize (object);
 }
@@ -1523,11 +1695,9 @@ conf_manager_get_title (ConfManager *conf)
 	gchar *str;
 
 	if (conf->working_file)
-		str = g_strdup_printf (_("Mergeant Version %s - %s"), VERSION,
-				       conf->working_file);
+		str = g_strdup_printf (_("%s - Mergeant"), conf->working_file);
 	else
-		str = g_strdup_printf (_("Mergeant Version %s - No file"),
-				       VERSION);
+		str = g_strdup (_("No file - Mergeant"));
 	return str;
 }
 
