@@ -69,7 +69,7 @@ sql_conn_open_cb (GtkWidget * widget, ConfManager * conf)
 {
 	gchar *str;
 
-	if (*(conf->srv->gda_datasource->str)) {
+	if (!conf->last_failed && *(conf->srv->gda_datasource->str)) {
 		/* we check if we have a password or not; if not, ask for one, except if we 
 		   have already entered it into the gnome-db login dialog */
 		if (conf->user_name && !conf->conn_open_requested &&
@@ -147,8 +147,10 @@ sql_conn_open_cb (GtkWidget * widget, ConfManager * conf)
 
 		}
 		server_access_open_connect (conf->srv);
+		conf->last_failed = (conf->srv->cnc == NULL);
 	}
 	else {
+		conf->last_failed = FALSE;
 		conf->conn_open_requested = TRUE;
 		options_config_cb (NULL, conf);
 	}
@@ -220,6 +222,20 @@ sql_conn_close_cb (GtkWidget * widget, ConfManager * conf)
 		if (!conf->working_file)
 			conf->close_after_saving = TRUE;
 	case 2: /* disconnect, no saving */
+		/* if we've got a pending transaction, ask the user what to do */
+		if (conf->current_transaction) {
+			if (gnome_db_yes_no_dialog (dialog,
+						    _("A transaction is pending. Would you like to "
+						      "commit the changes you did (select Yes) or "
+						      "discard them (select No)?"))) {
+				gda_connection_commit_transaction (conf->srv->cnc, conf->current_transaction);
+			} else
+				gda_connection_rollback_transaction (conf->srv->cnc, conf->current_transaction);
+
+			g_object_unref (G_OBJECT (conf->current_transaction));
+			conf->current_transaction = NULL;
+		}
+
 		conf->save_up_to_date = TRUE;
 		/* here we delay the closing of the connection because the user need to choose
 		   a filename to save data to */
@@ -232,6 +248,94 @@ sql_conn_close_cb (GtkWidget * widget, ConfManager * conf)
 	gtk_widget_destroy (dialog);
 }
 
+/* transaction-related callbacks */
+void
+sql_begin_trans_cb (GtkWidget * widget, ConfManager * conf)
+{
+	gchar *xname;
+	static guint counter = 0;
+
+	if (!gda_connection_supports (conf->srv->cnc, GDA_CONNECTION_FEATURE_TRANSACTIONS)) {
+		gnome_db_show_error (_("Underlying data source does not support transactions"));
+		return;
+	}
+
+	if (conf->current_transaction) {
+		gnome_db_show_error (_("A transaction is already running"));
+		return;
+	}
+
+	xname = g_strdup_printf ("mergeant_transaction_%d", counter++);
+	conf->current_transaction = gda_transaction_new (xname);
+
+	/* start the transaction on the server */
+	if (gda_connection_begin_transaction (conf->srv->cnc, conf->current_transaction)) {
+		gtk_image_set_from_stock (GTK_IMAGE (conf->icon_transaction),
+					  GNOME_DB_STOCK_WITHIN_TRANSACTION,
+					  GTK_ICON_SIZE_MENU);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_transaction,
+				      _("Transaction in progress"), _("Transaction in progress"));
+		gnome_appbar_set_status (GNOME_APPBAR (conf->appbar), _("Transaction started"));
+	} else {
+		g_object_unref (G_OBJECT (conf->current_transaction));
+		conf->current_transaction = NULL;
+	}
+
+	g_free (xname);
+}
+
+void
+sql_commit_trans_cb (GtkWidget * widget, ConfManager * conf)
+{
+	if (!gda_connection_supports (conf->srv->cnc, GDA_CONNECTION_FEATURE_TRANSACTIONS)) {
+		gnome_db_show_error (_("Underlying data source does not support transactions"));
+		return;
+	}
+
+	if (!conf->current_transaction) {
+		gnome_db_show_error (_("A transaction must be started before committing"));
+		return;
+	}
+
+	if (gda_connection_commit_transaction (conf->srv->cnc, conf->current_transaction)) {
+		g_object_unref (G_OBJECT (conf->current_transaction));
+		conf->current_transaction = NULL;
+
+		gtk_image_set_from_stock (GTK_IMAGE (conf->icon_transaction),
+					  GNOME_DB_STOCK_NO_TRANSACTION,
+					  GTK_ICON_SIZE_MENU);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_transaction,
+				      _("No transaction"), _("No transaction"));
+		gnome_appbar_set_status (GNOME_APPBAR (conf->appbar), _("Transaction saved"));
+	}
+}
+
+void
+sql_rollback_trans_cb (GtkWidget *widget, ConfManager * conf)
+{
+	if (!gda_connection_supports (conf->srv->cnc, GDA_CONNECTION_FEATURE_TRANSACTIONS)) {
+		gnome_db_show_error (_("Underlying data source does not support transactions"));
+		return;
+	}
+
+	if (!conf->current_transaction) {
+		gnome_db_show_error (_("A transaction must be started before cancelling"));
+		return;
+	}
+
+	if (gda_connection_rollback_transaction (conf->srv->cnc, conf->current_transaction))
+		gnome_appbar_set_status (GNOME_APPBAR (conf->appbar), _("Transaction aborted"));
+
+	g_object_unref (G_OBJECT (conf->current_transaction));
+	conf->current_transaction = NULL;
+
+	gtk_image_set_from_stock (GTK_IMAGE (conf->icon_transaction),
+				  GNOME_DB_STOCK_NO_TRANSACTION,
+				  GTK_ICON_SIZE_MENU);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_transaction,
+			      _("No transaction"), _("No transaction"));
+}
+
 /*
  * options_config_cb
  */
@@ -257,7 +361,7 @@ options_config_cb (GtkWidget * widget, ConfManager * conf)
 
                 if (conf->user_name)
                         g_free (conf->user_name);
-                conf->user_name = g_strdup (conf->srv->gda_datasource->str);
+                conf->user_name = g_strdup (conf->srv->user_name->str);
                 if (conf->user_passwd)
                         g_free (conf->user_passwd);
                 conf->user_passwd = g_strdup (conf->srv->password->str);
@@ -1232,10 +1336,45 @@ plugins_ch_combo_sel_changed_cb2 (GObject * object,
 }
 
 
+/*
+ * CBs for the View menu
+ */
 
+/* activate the tables page */
+void
+view_tables_tab_cb (GtkWidget * widget, ConfManager * conf)
+{
+	show_tables_page_cb (widget, NULL, NULL, conf);
+}
 
+/* activate the sequences page */
+void
+view_sequences_tab_cb (GtkWidget * widget, ConfManager * conf)
+{
+	if (conf->srv->features.sequences)
+		show_seqs_page_cb (widget, NULL, NULL, conf);
+}
 
+/* activate the queries page */
+void
+view_queries_tab_cb (GtkWidget * widget, ConfManager * conf)
+{
+	show_queries_page_cb (widget, NULL, NULL, conf);
+}
 
+/* activate the forms page */
+void
+view_forms_tab_cb (GtkWidget * widget, ConfManager * conf)
+{
+	show_forms_page_cb (widget, NULL, NULL, conf);
+}
+
+/* activate the SQL page */
+void
+view_sql_tab_cb (GtkWidget * widget, ConfManager * conf)
+{
+	show_sql_page_cb (widget, NULL, NULL, conf);
+}
 
 /*
  * printer_setup_cb
@@ -1257,13 +1396,23 @@ show_tables_page_cb (GtkWidget * shortcut, const gchar *label, const gchar *tool
 {
 	gtk_notebook_set_page (GTK_NOTEBOOK (conf->nb), 0);
 	gnome_db_gray_bar_set_text (GNOME_DB_GRAY_BAR (conf->title_bar), _("Tables and Views"));
+
+	gtk_image_set_from_stock (GTK_IMAGE (conf->icon_current_page), GNOME_DB_STOCK_TABLES, GTK_ICON_SIZE_MENU);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_current_page,
+			      _("Tables and Views page"), _("Tables and Views page"));
 }
 
 void
 show_seqs_page_cb (GtkWidget * wid, const gchar *label, const gchar *tooltip, ConfManager * conf)
 {
-	gtk_notebook_set_page (GTK_NOTEBOOK (conf->nb), 1);
-	gnome_db_gray_bar_set_text (GNOME_DB_GRAY_BAR (conf->title_bar), _("Sequences"));
+	if (conf->srv->features.sequences) {
+		gtk_notebook_set_page (GTK_NOTEBOOK (conf->nb), 1);
+		gnome_db_gray_bar_set_text (GNOME_DB_GRAY_BAR (conf->title_bar), _("Sequences"));
+
+		gtk_image_set_from_stock (GTK_IMAGE (conf->icon_current_page), "mergeant_sequences", GTK_ICON_SIZE_MENU);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_current_page,
+				      _("Sequences page"), _("Sequences page"));
+	}
 }
 
 void
@@ -1271,6 +1420,10 @@ show_queries_page_cb (GtkWidget * shortcut, const gchar *label, const gchar *too
 {
 	gtk_notebook_set_page (GTK_NOTEBOOK (conf->nb), 2);
 	gnome_db_gray_bar_set_text (GNOME_DB_GRAY_BAR (conf->title_bar), _("Queries"));
+
+	gtk_image_set_from_stock (GTK_IMAGE (conf->icon_current_page), "mergeant_queries", GTK_ICON_SIZE_MENU);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_current_page,
+			      _("Queries page"), _("Queries page"));
 }
 
 void
@@ -1278,6 +1431,10 @@ show_forms_page_cb (GtkWidget * shortcut, const gchar *label, const gchar *toolt
 {
 	gtk_notebook_set_page (GTK_NOTEBOOK (conf->nb), 3);
 	gnome_db_gray_bar_set_text (GNOME_DB_GRAY_BAR (conf->title_bar), _("Forms"));
+
+	gtk_image_set_from_stock (GTK_IMAGE (conf->icon_current_page), "mergeant_forms", GTK_ICON_SIZE_MENU);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_current_page,
+			      _("Forms page"), _("Forms page"));
 }
 
 void
@@ -1285,6 +1442,10 @@ show_sql_page_cb (GtkWidget * shortcut, const gchar *label, const gchar *tooltip
 {
 	gtk_notebook_set_page (GTK_NOTEBOOK (conf->nb), 4);
 	gnome_db_gray_bar_set_text (GNOME_DB_GRAY_BAR (conf->title_bar), _("SQL"));
+
+	gtk_image_set_from_stock (GTK_IMAGE (conf->icon_current_page), GNOME_DB_STOCK_SQL, GTK_ICON_SIZE_MENU);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (conf->icon_area_tooltips), conf->icon_current_page,
+			      _("SQL page"), _("SQL page"));
 }
 
 
@@ -1701,6 +1862,20 @@ quit_cb (GtkWidget * widget, ConfManager * conf)
 		if (!conf->working_file)
 			conf->quit_after_saving = TRUE;
 	case 2: /* just quit */
+		/* if we've got a pending transaction, ask the user what to do */
+		if (conf->current_transaction) {
+			if (gnome_db_yes_no_dialog (dialog,
+						    _("A transaction is pending. Would you like to "
+						      "commit the changes you did (select Yes) or "
+						      "discard them (select No)?"))) {
+				gda_connection_commit_transaction (conf->srv->cnc, conf->current_transaction);
+			} else
+				gda_connection_rollback_transaction (conf->srv->cnc, conf->current_transaction);
+
+			g_object_unref (G_OBJECT (conf->current_transaction));
+			conf->current_transaction = NULL;
+		}
+
 		conf->save_up_to_date = TRUE;
 		if (!conf->quit_after_saving)
 			quit_real (conf);
@@ -1749,9 +1924,15 @@ xml_parser_error_func (void *ctx, const char *msg, ...)
 	va_list args;
 	gchar *str;
 
+	va_start (args, msg);
+	str = g_strdup_vprintf (msg, args);
+	va_end (args);
+	g_print ("XML ERROR: %s\n", str);
+	g_free (str);
+
 	if (!has_error) {
 		GtkWidget *dlg;
-		conf = (ConfManager *) (ctxt->userData);
+		conf = CONF_MANAGER (ctxt->userData);
 		va_start (args, msg);
 		str = g_strdup_vprintf (msg, args);
 		va_end (args);
@@ -1987,17 +2168,24 @@ sql_server_conn_close_cb (GObject * wid, ConfManager * conf)
 
 
 /*
- * sql_server_catch_errors_cb
+ * sql_server_event_cb
  */
-void sql_server_catch_errors_cb (ServerAccess *srv, GdaConnection *cnc,
-				 GList * error_list, ConfManager * conf)
+void sql_server_event_cb (ServerAccess *srv, GdaConnection *cnc,
+			  GdaClientEvent event, GdaParameterList *params,
+			  ConfManager * conf)
 {
-	if (!conf->error_dlg) {
-		conf->error_dlg = gnome_db_error_dialog_new (_("Error Viewer"));
-		g_object_add_weak_pointer (G_OBJECT (conf->error_dlg), (gpointer) & (conf->error_dlg));
+	switch (event) {
+	case GDA_CLIENT_EVENT_ERROR :
+		if (!conf->error_dlg) {
+			conf->error_dlg = gnome_db_error_dialog_new (_("Error Viewer"));
+			g_object_add_weak_pointer (G_OBJECT (conf->error_dlg), (gpointer) & (conf->error_dlg));
+		}
+		else 
+			gdk_window_raise (GTK_WIDGET (conf->error_dlg)->window);
+
 		gnome_db_error_dialog_show_errors (GNOME_DB_ERROR_DIALOG (conf->error_dlg),
-						   error_list);
+						   gda_connection_get_errors (cnc));
+		break;
+	default :
 	}
-	else 
-		gdk_window_raise (GTK_WIDGET (conf->error_dlg)->window);
 }
