@@ -1,6 +1,7 @@
 /* mainpagetable.c
  *
  * Copyright (C) 1999 - 2002 Vivien Malerba
+ * Copyright (C) 2002 Rodrigo Moya
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,6 +20,7 @@
  */
 
 #include <config.h>
+#include <libgnomedb/gnome-db-grid.h>
 #include <libgnomedb/gnome-db-stock.h>
 #include <libgnomedb/gnome-db-util.h>
 #include "mainpagetable.h"
@@ -35,27 +37,23 @@ static void main_page_table_finalize (GObject *object);
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
 
-typedef struct
-{
+typedef struct {
 	ConfManager *conf;
 	DbTable     *table;
 	Query       *query;
 	QueryEnv    *env;
 	GtkWidget   *props_dlg;
 	GtkWidget   *exec_dlg;
-}
-Row_Data;
+} Row_Data;
 
 
 /*
  * static functions 
  */
-static void selection_made (GtkWidget * wid,
-			    gint row,
-			    gint column,
-			    GdkEventButton * event, gpointer data);
-static gint press_handler_event (GtkWidget * widget,
-				 GdkEventButton * event, MainPageTable * mpt);
+static void selection_made (GnomeDbGrid * grid, gint row, gpointer data);
+static void selection_cleared (GnomeDbGrid * grid, gpointer data);
+static void grid_double_clicked_cb (GnomeDbGrid * grid, gint row, MainPageTable * mpt);
+static void grid_create_popup_cb (GnomeDbGrid * grid, GtkWidget *menu, MainPageTable * mpt);
 static void remove_table_cb (GObject   * obj, MainPageTable * wid);
 static void edit_table_cb (GObject   * obj, MainPageTable * wid);
 static void view_records_button_cb (GObject   * obj, MainPageTable * mpt);
@@ -105,14 +103,17 @@ main_page_table_class_init (MainPageTableClass * class)
 static void
 main_page_table_init (MainPageTable * wid)
 {
-	GtkWidget *sw, *bb;
-	gint i;
+	GtkWidget *bb;
+	GdaDataModel *model;
 
 	/* Toolbar */
 	bb = gnome_db_new_toolbar_widget (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
 	gtk_box_pack_start (GTK_BOX (wid), bb, FALSE, TRUE, GNOME_PAD / 2);
 	gtk_widget_show (bb);
 
+	wid->new_table = gtk_toolbar_insert_stock (GTK_TOOLBAR (bb), GTK_STOCK_NEW,
+						   _("Create a new table"), NULL,
+						   NULL, NULL, -1); /* FIXME */
 	wid->remove_table = gtk_toolbar_insert_stock (GTK_TOOLBAR (bb), GTK_STOCK_DELETE,
 						      _("Drop selected table"), NULL,
 						      GTK_SIGNAL_FUNC (remove_table_cb), wid, -1);
@@ -122,37 +123,27 @@ main_page_table_init (MainPageTable * wid)
 	wid->view_table = gtk_toolbar_insert_stock (GTK_TOOLBAR (bb), GNOME_DB_STOCK_TABLES,
 						    _("View records for the selected table"), NULL,
 						    GTK_SIGNAL_FUNC (view_records_button_cb), wid, -1);
-	wid->new_table = gtk_toolbar_insert_stock (GTK_TOOLBAR (bb), GTK_STOCK_NEW,
-						   _("Create a new table"), NULL,
-						   NULL, NULL, -1); /* FIXME */
 
-	/* Scrolled Window for CList */
-	sw = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-					GTK_POLICY_AUTOMATIC,
-					GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start (GTK_BOX (wid), sw, TRUE, TRUE, GNOME_PAD);
-	gtk_widget_show (sw);
+	/* table list */
+	model = gda_data_model_array_new (4);
+	gda_data_model_set_column_title (model, 0, _("Name"));
+	gda_data_model_set_column_title (model, 1, _("Type"));
+	gda_data_model_set_column_title (model, 2, _("Owner"));
+	gda_data_model_set_column_title (model, 3, _("Comments"));
 
-	/* CList */
-	wid->clist = gtk_clist_new (4);
-	gtk_clist_set_column_title (GTK_CLIST (wid->clist), 0, _("Name"));
-	gtk_clist_set_column_title (GTK_CLIST (wid->clist), 1, _("Type"));
-	gtk_clist_set_column_title (GTK_CLIST (wid->clist), 2, _("Owner"));
-	gtk_clist_set_column_title (GTK_CLIST (wid->clist), 3, _("Comments"));
-	gtk_clist_set_selection_mode (GTK_CLIST (wid->clist),
-				      GTK_SELECTION_SINGLE);
-	for (i = 0; i < 4; i++)
-		gtk_clist_set_column_auto_resize (GTK_CLIST (wid->clist), i,
-						  TRUE);
-	gtk_clist_column_titles_show (GTK_CLIST (wid->clist));
-	gtk_clist_column_titles_passive (GTK_CLIST (wid->clist));
-	gtk_container_add (GTK_CONTAINER (sw), wid->clist);
-	gtk_widget_show (wid->clist);
-	g_signal_connect (G_OBJECT (wid->clist), "select_row",
+	wid->list = gnome_db_grid_new_with_model (model);
+	g_object_unref (G_OBJECT (model));
+	gnome_db_grid_set_selection_mode (GNOME_DB_GRID (wid->list), GTK_SELECTION_SINGLE);
+	gtk_box_pack_start (GTK_BOX (wid), wid->list, TRUE, TRUE, 2);
+	gtk_widget_show (wid->list);
+	g_signal_connect (G_OBJECT (wid->list), "row_selected",
 			  G_CALLBACK (selection_made), wid);
-	g_signal_connect (G_OBJECT (wid->clist), "button_press_event",
-			  G_CALLBACK (press_handler_event), wid);
+	g_signal_connect (G_OBJECT (wid->list), "selection_cleared",
+			  G_CALLBACK (selection_cleared), wid);
+	g_signal_connect (G_OBJECT (wid->list), "double_clicked",
+			  G_CALLBACK (grid_double_clicked_cb), wid);
+	g_signal_connect (G_OBJECT (wid->list), "create_popup_menu",
+			  G_CALLBACK (grid_create_popup_cb), wid);
 
 	wid->sel_row = -1;
 }
@@ -190,17 +181,15 @@ static void main_page_table_finalize (GObject *object)
 
 static void database_added_cb (ConfManager *conf, Database *db, MainPageTable * mpt)
 {
-	g_signal_connect (G_OBJECT (conf->db),
-			    "table_created",
-			    G_CALLBACK (main_page_table_add_cb),
-			    mpt);
-	g_signal_connect (G_OBJECT (conf->db),
-			    "table_dropped",
-			    G_CALLBACK (main_page_table_drop_cb),
-			    mpt);
+	g_signal_connect (G_OBJECT (conf->db), "table_created",
+			  G_CALLBACK (main_page_table_add_cb),
+			  mpt);
+	g_signal_connect (G_OBJECT (conf->db), "table_dropped",
+			  G_CALLBACK (main_page_table_drop_cb),
+			  mpt);
 	g_signal_connect (G_OBJECT (conf->db), "updated",
-			    G_CALLBACK (main_page_db_updated_cb),
-			    mpt);
+			  G_CALLBACK (main_page_db_updated_cb),
+			  mpt);
 }
 
 static void database_removed_cb (ConfManager *conf, Database *db, MainPageTable * mpt)
@@ -215,7 +204,6 @@ main_page_table_initialize (MainPageTable * wid)
 	gtk_widget_set_sensitive (wid->remove_table, FALSE);
 	gtk_widget_set_sensitive (wid->edit_table, FALSE);
 	gtk_widget_set_sensitive (wid->view_table, FALSE);
-	gtk_widget_set_sensitive (wid->new_table, FALSE);
 	conf_manager_register_sensitive_on_connect (wid->conf,
 						    GTK_WIDGET (wid->new_table));
 }
@@ -225,8 +213,7 @@ main_page_table_set_data_dlg (MainPageTable * mpt, guint row, GtkWidget * dlg)
 {
 	Row_Data *rdata = NULL;
 
-	rdata = (Row_Data *) gtk_clist_get_row_data (GTK_CLIST (mpt->clist),
-						     row);
+	rdata = (Row_Data *) gnome_db_grid_get_row_data (GNOME_DB_GRID (mpt->list), row);
 	if (rdata)
 		rdata->props_dlg = dlg;
 }
@@ -236,8 +223,7 @@ main_page_table_get_data_dlg (MainPageTable * mpt, guint row)
 {
 	Row_Data *rdata = NULL;
 
-	rdata = (Row_Data *) gtk_clist_get_row_data (GTK_CLIST (mpt->clist),
-						     row);
+	rdata = (Row_Data *) gnome_db_grid_get_row_data (GNOME_DB_GRID (mpt->list), row);
 	if (rdata)
 		return rdata->props_dlg;
 	else
@@ -251,9 +237,10 @@ main_page_table_get_row (MainPageTable * mpt, DbTable * table)
 	gboolean found = FALSE;
 	Row_Data *rdata;
 
-	while ((i < GTK_CLIST (mpt->clist)->rows) && !found) {
-		rdata = gtk_clist_get_row_data (GTK_CLIST (mpt->clist), i);
-		if (rdata && (rdata->table == table))
+	while ((i < gda_data_model_get_n_rows (gnome_db_grid_get_model (GNOME_DB_GRID (mpt->list)))) && !found) {
+		rdata = gnome_db_grid_get_row_data (GNOME_DB_GRID (mpt->list), i);
+		g_assert (rdata);
+		if (rdata->table == table)
 			found = TRUE;
 		else
 			i++;
@@ -267,8 +254,7 @@ main_page_table_get_row (MainPageTable * mpt, DbTable * table)
 
 
 static void
-selection_made (GtkWidget * wid,
-		gint row, gint column, GdkEventButton * event, gpointer data)
+selection_made (GnomeDbGrid * grid, gint row, gpointer data)
 {
 	MainPageTable *mpt = MAIN_PAGE_TABLE (data);
 
@@ -280,57 +266,53 @@ selection_made (GtkWidget * wid,
 	gtk_widget_set_sensitive (mpt->view_table, TRUE);
 }
 
-
-static gint
-press_handler_event (GtkWidget * widget,
-		     GdkEventButton * event, MainPageTable * mpt)
+static void
+selection_cleared (GnomeDbGrid * grid, gpointer data)
 {
-	gint row, col;
+	MainPageTable *mpt = MAIN_PAGE_TABLE (data);
 
-	if (GTK_IS_CLIST (widget)) {
-		/* setting the right selection */
-		gtk_clist_get_selection_info (GTK_CLIST (widget),
-					      event->x, event->y, &row, &col);
-		if ((mpt->sel_row != row) && (event->button != 1))
-			gtk_clist_select_row (GTK_CLIST (widget), row, col);
+	mpt->sel_row = -1;
+	gtk_widget_set_sensitive (mpt->remove_table, FALSE);
+	gtk_widget_set_sensitive (mpt->edit_table, FALSE);
+	gtk_widget_set_sensitive (mpt->view_table, FALSE);
+}
 
+static void
+grid_double_clicked_cb (GnomeDbGrid * grid, gint row, MainPageTable * mpt)
+{
+	if (GNOME_DB_IS_GRID (grid)) {
+		Row_Data *rd;
 
-		/* depending on what the clicks were */
-		if (event->type == GDK_2BUTTON_PRESS) {
-			Row_Data *rd;
-
-			rd = (Row_Data *)
-				gtk_clist_get_row_data (GTK_CLIST (widget),
-							row);
-			view_records_button_cb (NULL, mpt);
-			/*edit_table_cb(NULL, mpt); */
-			return TRUE;
-		}
-
-		if (event->button == 3) {
-			GtkWidget *menu, *wid;
-
-			menu = gtk_menu_new ();
-			wid = gtk_menu_item_new_with_label (_("Properties"));
-			g_signal_connect (G_OBJECT (wid), "activate",
-					    G_CALLBACK (edit_table_cb),
-					    mpt);
-			gtk_menu_append (GTK_MENU (menu), wid);
-			gtk_widget_show (wid);
-			wid = gtk_menu_item_new_with_label (_
-							    ("View records"));
-			g_signal_connect (G_OBJECT (wid), "activate",
-					  G_CALLBACK (view_records_button_cb), mpt);
-			gtk_menu_append (GTK_MENU (menu), wid);
-			gtk_widget_show (wid);
-			gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL,
-					NULL, event->button, event->time);
-			/* Tell calling code that we have handled this event */
-			return TRUE;
-		}
+		rd = (Row_Data *) gnome_db_grid_get_row_data (grid, row);
+		view_records_button_cb (NULL, mpt);
 	}
+}
 
-	return FALSE;
+static void
+grid_create_popup_cb (GnomeDbGrid * grid, GtkWidget * menu, MainPageTable * mpt)
+{
+	GtkWidget *wid;
+
+	wid = gtk_separator_menu_item_new ();
+	gtk_menu_append (GTK_MENU (menu), wid);
+	gtk_widget_show (wid);
+	
+	wid = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
+	g_signal_connect (G_OBJECT (wid), "activate",
+			  G_CALLBACK (edit_table_cb),
+			  mpt);
+	gtk_menu_append (GTK_MENU (menu), wid);
+	gtk_widget_show (wid);
+	if (mpt->sel_row == -1)
+		gtk_widget_set_sensitive (wid, FALSE);
+
+	wid = gtk_menu_item_new_with_label (_("View records"));
+	g_signal_connect (G_OBJECT (wid), "activate",
+			  G_CALLBACK (view_records_button_cb), mpt);
+	gtk_menu_append (GTK_MENU (menu), wid);
+	gtk_widget_show (wid);
+	if (mpt->sel_row == -1)
+		gtk_widget_set_sensitive (wid, FALSE);
 }
 
 static void remove_table_dlg_response_cb (GtkDialog *dlg, gint reply, MainPageTable * wid);
@@ -345,8 +327,7 @@ remove_table_cb (GObject   * obj, MainPageTable * wid)
 
 	if (wid->sel_row >= 0)
 		rdata = (Row_Data *)
-			gtk_clist_get_row_data (GTK_CLIST (wid->clist),
-						wid->sel_row);
+			gnome_db_grid_get_row_data (GNOME_DB_GRID (wid->list), wid->sel_row);
 	if (rdata) {
 		if (rdata->table->is_view)
 			str = g_strdup_printf (_("Do you really wish to drop the view '%s'?"),
@@ -380,9 +361,7 @@ remove_table_dlg_response_cb (GtkDialog *dlg, gint reply, MainPageTable * wid)
 	gchar *query;
 
 	if (wid->sel_row >= 0)
-		rdata = (Row_Data *)
-			gtk_clist_get_row_data (GTK_CLIST (wid->clist),
-						wid->sel_row);
+		rdata = (Row_Data *) gnome_db_grid_get_row_data (GNOME_DB_GRID (wid->list), wid->sel_row);
 
 	switch (reply) {
 	case GTK_RESPONSE_YES:
@@ -429,9 +408,7 @@ edit_table_cb (GObject   * obj, MainPageTable * wid)
 	GtkWidget *dlg;
 
 	if (wid->sel_row >= 0)
-		rdata = (Row_Data *)
-			gtk_clist_get_row_data (GTK_CLIST (wid->clist),
-						wid->sel_row);
+		rdata = (Row_Data *) gnome_db_grid_get_row_data (GNOME_DB_GRID (wid->list), wid->sel_row);
 	if (rdata) {
 		dlg = table_edit_dialog_new (wid->conf, rdata->table);
 		if (dlg)
@@ -446,9 +423,7 @@ view_records_button_cb (GObject   * obj, MainPageTable * mpt)
 	gchar *str;
 
 	if (mpt->sel_row >= 0)
-		rdata = (Row_Data *)
-			gtk_clist_get_row_data (GTK_CLIST (mpt->clist),
-						mpt->sel_row);
+		rdata = (Row_Data *) gnome_db_grid_get_row_data (GNOME_DB_GRID (mpt->list), mpt->sel_row);
 	if (rdata) {
 		QueryExec *qx;
 
@@ -474,7 +449,7 @@ view_records_button_cb (GObject   * obj, MainPageTable * mpt)
 				query_env_set_modif_table (rdata->env, rdata->table);
 				rdata->env->actions =
 					QUERY_ACTION_INSERT | QUERY_ACTION_REFRESH |
-					 QUERY_ACTION_DELETE |
+					QUERY_ACTION_DELETE |
 					QUERY_ACTION_COMMIT | QUERY_ACTION_FIRST |
 					QUERY_ACTION_LAST | QUERY_ACTION_PREV |
 					QUERY_ACTION_NEXT;
@@ -503,17 +478,18 @@ static void
 main_page_table_conn_close_cb (GObject   * obj, MainPageTable * mpt)
 {
 	Row_Data *rdata;
-	gint i, nb = GTK_CLIST (mpt->clist)->rows;
+	gint i, nb = gda_data_model_get_n_rows (gnome_db_grid_get_model (GNOME_DB_GRID (mpt->list)));
 
-	gtk_clist_freeze (GTK_CLIST (mpt->clist));
-	for (i = 0; i < nb; i++) {
-		rdata = (Row_Data *)
-			gtk_clist_get_row_data (GTK_CLIST (mpt->clist), 0);
+	/* Because when the connexion is closed, there should be no DbTable left, 
+	   normally nb sould be equal to 0, but just in case */
+	for (i = nb - 1; i >= 0; i++) {
+		rdata = (Row_Data *) gnome_db_grid_get_row_data (GNOME_DB_GRID (mpt->list), i);
 		drop_associated_query (rdata);
 		g_free (rdata);
-		gtk_clist_remove (GTK_CLIST (mpt->clist), 0);
+
+		gda_data_model_remove_row (gnome_db_grid_get_model (GNOME_DB_GRID (mpt->list)), 
+					   gda_data_model_get_row (gnome_db_grid_get_model (GNOME_DB_GRID (mpt->list)), i));
 	}
-	gtk_clist_thaw (GTK_CLIST (mpt->clist));
 	gtk_widget_set_sensitive (mpt->remove_table, FALSE);
 	gtk_widget_set_sensitive (mpt->edit_table, FALSE);
 	gtk_widget_set_sensitive (mpt->view_table, FALSE);
@@ -528,63 +504,54 @@ main_page_table_conn_close_cb (GObject   * obj, MainPageTable * mpt)
 */
 static void comments_table_changed_cb (DbTable * table, Row_Data * rdata);
 static void
-main_page_table_add_cb (GObject   * obj, DbTable * table,
-			MainPageTable * wid)
+main_page_table_add_cb (GObject * obj, DbTable * table, MainPageTable * wid)
 {
-	gchar *col[4];
-	gchar *str;
-	gint i = 0, j;
-	gboolean stop = FALSE;
+	GList *cols = NULL;
 	Row_Data *rdata;
 
-	col[0] = g_strdup (table->name);
+	cols = g_list_append (cols, gda_value_new_string (table->name));
 	if (table->is_view)
-		col[1] = g_strdup (_("View"));
+		cols = g_list_append (cols, gda_value_new_string (_("View")));
 	else
-		col[1] = g_strdup (_("Table"));
-	col[2] = g_strdup (table->owner);
-	if (table->comments)
-		col[3] = g_strdup (table->comments);
-	else
-		col[3] = g_strdup ("");
-	/* where to put it ? */
-	while ((i < GTK_CLIST (wid->clist)->rows) && !stop) {
-		gtk_clist_get_text (GTK_CLIST (wid->clist), i, 0, &str);
-		if (strcmp (str, table->name) > 0) {
-			stop = TRUE;
-		}
-		else
-			i++;
-	}
+		cols = g_list_append (cols, gda_value_new_string (_("Table")));
+	cols = g_list_append (cols, gda_value_new_string (table->owner));
+	cols = g_list_append (cols, gda_value_new_string (table->comments));
 
-	gtk_clist_insert (GTK_CLIST (wid->clist), i, col);
-	for (j = 0; j < 4; j++)
-		g_free (col[j]);
+	gda_data_model_append_row (gnome_db_grid_get_model (GNOME_DB_GRID (wid->list)), cols);
+	g_list_foreach (cols, (GFunc) gda_value_free, NULL);
+	g_list_free (cols);
 
-	rdata = (Row_Data *) g_malloc (sizeof (Row_Data));
+	rdata = g_new0 (Row_Data, 1);
 	rdata->conf = wid->conf;
 	rdata->table = table;
 	rdata->query = NULL;
 	rdata->env = NULL;
 	rdata->props_dlg = NULL;
 	rdata->exec_dlg = NULL;
-	gtk_clist_set_row_data (GTK_CLIST (wid->clist), i, rdata);
+	gnome_db_grid_set_row_data (GNOME_DB_GRID (wid->list),
+				    gda_data_model_get_n_rows (gnome_db_grid_get_model (GNOME_DB_GRID (wid->list))) - 1,
+				    rdata);
 	g_signal_connect (G_OBJECT (table), "comments_changed",
-			    G_CALLBACK (comments_table_changed_cb),
-			    rdata);
+			  G_CALLBACK (comments_table_changed_cb),
+			  rdata);
+
+	g_print ("ADDED TABLE %s, rdata=%p\n", table->name, rdata);
 }
 
 static void
 comments_table_changed_cb (DbTable * table, Row_Data * rdata)
 {
 	gint row;
-	GtkWidget *clist;
+	GtkWidget *list;
 
-	clist = MAIN_PAGE_TABLE (rdata->conf->tables_page)->clist;
-	row = gtk_clist_find_row_from_data (GTK_CLIST (clist), rdata);
-	if (row >= 0)
-		gtk_clist_set_text (GTK_CLIST (clist), row, 3,
-				    table->comments);
+	list = MAIN_PAGE_TABLE (rdata->conf->tables_page)->list;
+	row = gnome_db_grid_find_row_from_data (GNOME_DB_GRID (list), rdata);
+	if (row >= 0) {
+		gda_value_set_string (
+			gda_data_model_get_value_at (gnome_db_grid_get_model (GNOME_DB_GRID (list)), 3, row),
+			table->comments);
+		gda_data_model_row_updated (gnome_db_grid_get_model (GNOME_DB_GRID (list)), row);
+	}
 }
 
 
@@ -595,42 +562,47 @@ comments_table_changed_cb (DbTable * table, Row_Data * rdata)
 */
 
 static void
-main_page_table_drop_cb (GObject   * obj, DbTable * table,
-			 MainPageTable * wid)
+main_page_table_drop_cb (GObject   * obj, DbTable * table, MainPageTable * wid)
 {
-	gchar *str;
 	gint i = 0;
 	gboolean found = FALSE;
 	Row_Data *rdata;
+	GdaDataModel *model;
 
-	while ((i < GTK_CLIST (wid->clist)->rows) && !found) {
-		gtk_clist_get_text (GTK_CLIST (wid->clist), i, 0, &str);
-		if (strcmp (str, table->name) == 0) {
+	g_print ("REMOVED TABLE %s\n", table->name);
+
+	model = gnome_db_grid_get_model (GNOME_DB_GRID (wid->list));
+	while ((i < gda_data_model_get_n_rows (model)) && !found) {
+		rdata = (Row_Data *) gnome_db_grid_get_row_data (GNOME_DB_GRID (wid->list), i);
+		g_assert (rdata);
+
+		if (rdata->table == table) {
 			found = TRUE;
-			rdata = (Row_Data *)
-				gtk_clist_get_row_data (GTK_CLIST (wid->clist), i);
 			/* the associated query is not valid anymore */
 			drop_associated_query (rdata);
 			g_free (rdata);
-			gtk_clist_remove (GTK_CLIST (wid->clist), i);
+			gda_data_model_remove_row (model,
+						   gda_data_model_get_row (model, i));
 			if (wid->sel_row == i) {
 				wid->sel_row = -1;
 				/* un sensitive buttons */
-				gtk_widget_set_sensitive (wid->remove_table,
-							  FALSE);
-				gtk_widget_set_sensitive (wid->edit_table,
-							  FALSE);
-				gtk_widget_set_sensitive (wid->view_table,
-							  FALSE);
+				gtk_widget_set_sensitive (wid->remove_table, FALSE);
+				gtk_widget_set_sensitive (wid->edit_table, FALSE);
+				gtk_widget_set_sensitive (wid->view_table, FALSE);
 			}
 		}
 		i++;
 	}
-}
 
+	if (!found)
+		g_warning ("main_page_table_drop_cb: table '%s' not found!\n", table->name);
+}
+ 
 static void
 drop_associated_query (Row_Data * rdata)
 {
+	g_return_if_fail (rdata);
+
 	if (rdata->env) 
 		/* don't free the env, that will be done by the query which holds
 		   a ref on it. */
@@ -648,21 +620,23 @@ main_page_db_updated_cb (Database * db, MainPageTable * mpt)
 {
 	gint row;
 	Row_Data *rd;
-	for (row = 0; row < GTK_CLIST (mpt->clist)->rows; row++) {
-		rd = (Row_Data *)
-			gtk_clist_get_row_data (GTK_CLIST (mpt->clist), row);
-		if (rd->table->is_view)
-			gtk_clist_set_text (GTK_CLIST (mpt->clist), row, 1,
-					    _("View"));
-		else
-			gtk_clist_set_text (GTK_CLIST (mpt->clist), row, 1,
-					    _("Table"));
-		if (rd->table->owner)
-			gtk_clist_set_text (GTK_CLIST (mpt->clist), row, 2,
-					    rd->table->owner);
-		else
-			gtk_clist_set_text (GTK_CLIST (mpt->clist), row, 2,
-					    "");
+	GdaDataModel *model;
+
+	model = gnome_db_grid_get_model (GNOME_DB_GRID (mpt->list));
+	for (row = 0; row < gda_data_model_get_n_rows (model); row++) {
+		rd = (Row_Data *) gnome_db_grid_get_row_data (GNOME_DB_GRID (mpt->list), row);
+		g_assert (rd);
+
+		if (rd->table->is_view) {
+			gda_value_set_string (
+				gda_data_model_get_value_at (model, 1, row), _("View"));
+		} 
+		else {
+			gda_value_set_string (gda_data_model_get_value_at (model, 1, row), _("Table"));
+		}
+
+		gda_value_set_string (gda_data_model_get_value_at (model, 2, row), rd->table->owner);
+		gda_data_model_row_updated (model, row);
 
 		/* the associated query is not valid anymore */
 		drop_associated_query (rd);
