@@ -297,7 +297,7 @@ query_cond_add_to_node (QueryCond *qc, QueryCond *child, gint pos)
 	g_return_if_fail (child && IS_QUERY_COND (child));
 	g_return_if_fail (child != qc);
 	g_return_if_fail (qc->priv->cond_type != QUERY_COND_COND);
-	g_return_if_fail (!g_slist_find (qc->priv->children, child));
+	g_return_if_fail (! g_slist_find (qc->priv->children, child));
 	g_return_if_fail (! child->priv->parent);
 	
 	/* add the child */
@@ -702,7 +702,6 @@ query_cond_get_op_sql (QueryCondOpType op_type)
 	return str;
 }
 
-
 static QueryCondOpType
 query_cond_get_op_enum (const gchar *optype)
 {
@@ -974,6 +973,16 @@ query_cond_get_op_right2 (QueryCond *qc)
 	return qc->priv->right_op2;
 }
 
+const gchar *
+query_cond_render_op_sql (QueryCond *qc)
+{
+	g_return_val_if_fail (qc && IS_QUERY_COND (qc), NULL);
+	g_return_val_if_fail (qc->priv->cond_type == QUERY_COND_COND, NULL);
+
+	return query_cond_get_op_sql (qc->priv->op_type);
+}
+
+
 
 #ifdef debug
 /* structure debug function */
